// (c) 2006 Martin Aumueller <aumuell@reserv.at>
// modified 2006 Daniel Faust <hessijames@gmail.com>
// See COPYING file for licensing information

#ifndef TAGLIB_MONKEYSAUDIOFILETYPERESOLVER_H
#define TAGLIB_MONKEYSAUDIOFILETYPERESOLVER_H

#include <taglib/tfile.h>
#include <taglib/fileref.h>


class MonkeysAudioFileTypeResolver : public TagLib::FileRef::FileTypeResolver
{
    TagLib::File *createFile(const char *fileName,
            bool readAudioProperties,
            TagLib::AudioProperties::ReadStyle audioPropertiesStyle) const;
};

#endif
