
#include "pluginloaderbase.h"

PluginLoaderBase::PluginLoaderBase()
{}

PluginLoaderBase::~PluginLoaderBase()
{}

void PluginLoaderBase::unloadAll()
{}

void PluginLoaderBase::unload( TQString pluginName )
{}

void PluginLoaderBase::remove( TQString pluginName )
{}

#include "pluginloaderbase.moc"
