

#ifndef CONFIGPAGEBASE_H
#define CONFIGPAGEBASE_H

#include <tqwidget.h>

/**
 * @short The base for all pages of the config dialog
 * @author Daniel Faust <hessijames@gmail.com>
 * @version 0.3
 */
class ConfigPageBase : public TQWidget
{
    TQ_OBJECT
  
public:
    /**
     * Constructor
     */
    ConfigPageBase( TQWidget *parent=0, const char *name=0 );

    /**
     * Destructor
     */
    virtual ~ConfigPageBase();

public slots:
    virtual void resetDefaults();
    virtual void saveSettings();
    void cfgChanged();

signals:
    void configChanged();

};

#endif // CONFIGPAGEBASE_H

