
#include "configpagebase.h"

ConfigPageBase::ConfigPageBase( TQWidget *parent, const char *name )
    : TQWidget( parent, name )
{}

ConfigPageBase::~ConfigPageBase()
{}

void ConfigPageBase::resetDefaults()
{}

void ConfigPageBase::saveSettings()
{}

void ConfigPageBase::cfgChanged()
{
    emit configChanged();
}

#include "configpagebase.moc"
